#include "../../include/package.h"
#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdlib.h>
#include <string.h>

lm_ctx_update_list_t *lm_ctx_update_list(lm_ctx_t *ctx){
  if(NULL == ctx){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_ctx_update_list_t *list = malloc(sizeof(lm_ctx_update_list_t));
  lm_pkg_t cur, *new = NULL;
  
  bzero(list, sizeof(lm_ctx_update_list_t));
  lm_package_init(&cur);

  while(lm_ctx_database_next(ctx, &cur)){
    if((new = lm_ctx_pool_find(ctx, cur.name, NULL)) == NULL)
      continue;

    if(eq(new->version, cur.version))
      continue;

    if(NULL == list->packages)
      list->packages = malloc(sizeof(lm_pkg_t));
    else
      list->packages = realloc(list->packages, (list->count+1)*sizeof(lm_pkg_t));

    lm_package_copy(&list->packages[list->count++], &cur);
  }

  lm_ctx_database_next_free(ctx, &cur);
  return list;
}

lm_pkg_t *lm_ctx_update_list_next(lm_ctx_update_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(NULL == list->packages)
    return NULL;

  if(NULL == list->cur){
    list->cur = &list->packages[0];
    return list->cur;
  }

  for(int i = 0; i < list->count-1; i++){
    if(list->cur != &list->packages[i])
      continue;
    list->cur = &list->packages[i+1];
    return list->cur;
  }

  list->cur = NULL;
  return list->cur;
}

void lm_ctx_update_list_free(lm_ctx_update_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return;
  }

  if(NULL != list->packages){
    for(int i = 0; i < list->count; i++)
      lm_package_free(&list->packages[i]);
    free(list->packages);
  }
 
  free(list);
}

lm_pkg_t *lm_ctx_update(lm_ctx_t *ctx, lm_pkg_t *pkg){
  if(NULL == ctx || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pkg_t *new = NULL;

  if((new = lm_ctx_pool_find(ctx, pkg->name, NULL)) == NULL){
    lm_error_set(LM_ERR_PkgNotFound);
    return NULL;
  }

  if(eq(new->version, pkg->version)){
    lm_error_set(LM_ERR_PkgUpToDate);
    return NULL;
  }

  return new;
}
