#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <ini.h>

int lm_package_data_handler(void *data, const char *_section, const char *_key, const char *_value) {
  char      *section = (char *)_section, *value = (char *)_value, *key = (char *)_key;
  lm_pkg_t *pkg = data;

  if(NULL == pkg->name){
    if(!package_name_valid(section))
      return 0;
    pkg->name = strdup(section);
  }

  else if(!eq(pkg->name, section))
    return 0;

  if(eq(key, PKG_DATA_DESC))
    pkg->desc = strdup(value);
  
  else if(eq(key, PKG_DATA_VERSION)){
    if(!package_version_valid(value))
      return 0;
    pkg->version = strdup(value);
  }
  
  else if(eq(key, PKG_DATA_SIZE))
    pkg->size = atol(value);
  
  else if(eq(key, PKG_DATA_DEPENDS)){
    if(!lm_package_depend_add(pkg, value))
      return 0;
  }

  else if(eq(key, PKG_DATA_KEEPS)){
    if(!lm_package_keep_add(pkg, value))
      return 0;
  }

  return 1;
}

bool lm_package_data_load(lm_pkg_t *pkg, char *file){
  lm_package_data_free(pkg);

  if (ini_parse(file, lm_package_data_handler, pkg) < 0) {
    lm_error_set(LM_ERR_PkgDataBad);
    return false;
  }

  return true;
}

void lm_package_data_free(lm_pkg_t *pkg){
  free(pkg->desc);
  free(pkg->name);
  free(pkg->version);

  lm_package_depend_free(pkg);
  lm_package_keep_free(pkg);
}
