#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/types.h"
#include "../../include/util.h"

#include <stdlib.h>
#include <string.h>

bool lm_package_path_set_archive(lm_pkg_t *pkg, char *archive_path){
  free(pkg->paths.archive);
  pkg->paths.archive = NULL;

  if(NULL == archive_path)
    return true;

  if(is_dir(archive_path))
    return false;

  pkg->paths.archive = strdup(archive_path);
  return true;
}

bool lm_package_path_set_signature(lm_pkg_t *pkg, char *signature_path){
  free(pkg->paths.signature);
  pkg->paths.signature = NULL;

  if(NULL == signature_path)
    return true;

  if(is_dir(signature_path))
    return false;

  pkg->paths.signature = strdup(signature_path);
  return true;
}

bool lm_package_path_is_empty(lm_pkg_t *pkg){
  return NULL == pkg->paths.signature ||
         NULL == pkg->paths.archive;
}

bool lm_package_path_copy(lm_pkg_t *dst, lm_pkg_t *src){
  if(NULL == dst || NULL == src){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(NULL != src->paths.archive)
    dst->paths.archive = strdup(src->paths.archive);

  if(NULL != src->paths.signature)
    dst->paths.signature = strdup(src->paths.signature);

  return true;
}

void lm_package_path_free(lm_pkg_t *pkg){
  free(pkg->paths.signature);
  free(pkg->paths.archive);
}
